@@ -6,6 +6,7 @@
 
 from __future__ import annotations
 
+import sys
 import typing
 from collections.abc import Iterator
 from functools import partial
@@ -27,16 +28,24 @@ from astroid.nodes.node_classes import (
     Const,
     JoinedStr,
     Name,
-    NodeNG,
     Subscript,
     Tuple,
 )
 from astroid.nodes.scoped_nodes import ClassDef, FunctionDef
 from astroid.util import Uninferable
 
-TYPING_NAMEDTUPLE_BASENAMES = {"NamedTuple", "typing.NamedTuple"}
+if sys.version_info >= (3, 8):
+    from typing import Final
+else:
+    from typing_extensions import Final
+
 TYPING_TYPEVARS = {"TypeVar", "NewType"}
-TYPING_TYPEVARS_QUALIFIED = {"typing.TypeVar", "typing.NewType"}
+TYPING_TYPEVARS_QUALIFIED: Final = {
+    "typing.TypeVar",
+    "typing.NewType",
+    "typing_extensions.TypeVar",
+}
+TYPING_TYPEDDICT_QUALIFIED: Final = {"typing.TypedDict", "typing_extensions.TypedDict"}
 TYPING_TYPE_TEMPLATE = """
 class Meta(type):
     def __getitem__(self, item):
@@ -113,7 +122,7 @@ def looks_like_typing_typevar_or_newtype(node) -> bool:
 
 
 def infer_typing_typevar_or_newtype(node, context_itton=None):
-    """Infer a typing.TypeVar(...) or typing.NewType(...) call"""
+    """Infer a typing.TypeVar(...) or typing.NewType(...) call."""
     try:
         func = next(node.func.infer(context=context_itton))
     except (InferenceError, StopIteration) as exc:
@@ -133,7 +142,7 @@ def infer_typing_typevar_or_newtype(node, context_itton=None):
 
 
 def _looks_like_typing_subscript(node) -> bool:
-    """Try to figure out if a Subscript node *might* be a typing-related subscript"""
+    """Try to figure out if a Subscript node *might* be a typing-related subscript."""
     if isinstance(node, Name):
         return node.name in TYPING_MEMBERS
     if isinstance(node, Attribute):
@@ -146,7 +155,7 @@ def _looks_like_typing_subscript(node) -> bool:
 def infer_typing_attr(
     node: Subscript, ctx: context.InferenceContext | None = None
 ) -> Iterator[ClassDef]:
-    """Infer a typing.X[...] subscript"""
+    """Infer a typing.X[...] subscript."""
     try:
         value = next(node.value.infer())  # type: ignore[union-attr] # value shouldn't be None for Subscript.
     except (InferenceError, StopIteration) as exc:
@@ -186,7 +195,7 @@ def _looks_like_typedDict(  # pylint: disable=invalid-name
     node: FunctionDef | ClassDef,
 ) -> bool:
     """Check if node is TypedDict FunctionDef."""
-    return node.qname() in {"typing.TypedDict", "typing_extensions.TypedDict"}
+    return node.qname() in TYPING_TYPEDDICT_QUALIFIED
 
 
 def infer_old_typedDict(  # pylint: disable=invalid-name
@@ -216,6 +225,7 @@ def infer_typedDict(  # pylint: disable=invalid-name
 def _looks_like_typing_alias(node: Call) -> bool:
     """
     Returns True if the node corresponds to a call to _alias function.
+
     For example :
 
     MutableSet = _alias(collections.abc.MutableSet, T)
@@ -233,9 +243,7 @@ def _looks_like_typing_alias(node: Call) -> bool:
 
 
 def _forbid_class_getitem_access(node: ClassDef) -> None:
-    """
-    Disable the access to __class_getitem__ method for the node in parameters
-    """
+    """Disable the access to __class_getitem__ method for the node in parameters."""
 
     def full_raiser(origin_func, attr, *args, **kwargs):
         """
@@ -321,7 +329,6 @@ def _looks_like_special_alias(node: Call) -> bool:
     PY37: Tuple = _VariadicGenericAlias(tuple, (), inst=False, special=True)
     PY39: Tuple = _TupleType(tuple, -1, inst=False, name='Tuple')
 
-
     PY37: Callable = _VariadicGenericAlias(collections.abc.Callable, (), special=True)
     PY39: Callable = _CallableType(collections.abc.Callable, 2)
     """
@@ -372,36 +379,6 @@ def infer_special_alias(
     return iter([class_def])
 
 
-def _looks_like_typing_cast(node: Call) -> bool:
-    return isinstance(node, Call) and (
-        isinstance(node.func, Name)
-        and node.func.name == "cast"
-        or isinstance(node.func, Attribute)
-        and node.func.attrname == "cast"
-    )
-
-
-def infer_typing_cast(
-    node: Call, ctx: context.InferenceContext | None = None
-) -> Iterator[NodeNG]:
-    """Infer call to cast() returning same type as casted-from var"""
-    if not isinstance(node.func, (Name, Attribute)):
-        raise UseInferenceDefault
-
-    try:
-        func = next(node.func.infer(context=ctx))
-    except (InferenceError, StopIteration) as exc:
-        raise UseInferenceDefault from exc
-    if (
-        not isinstance(func, FunctionDef)
-        or func.qname() != "typing.cast"
-        or len(node.args) != 2
-    ):
-        raise UseInferenceDefault
-
-    return node.args[1].infer(context=ctx)
-
-
 AstroidManager().register_transform(
     Call,
     inference_tip(infer_typing_typevar_or_newtype),
@@ -410,9 +387,6 @@ AstroidManager().register_transform(
 AstroidManager().register_transform(
     Subscript, inference_tip(infer_typing_attr), _looks_like_typing_subscript
 )
-AstroidManager().register_transform(
-    Call, inference_tip(infer_typing_cast), _looks_like_typing_cast
-)
 
 if PY39_PLUS:
     AstroidManager().register_transform(
