@@ -2,7 +2,7 @@
 # For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 # Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
-"""this module contains a set of functions to handle python protocols for nodes
+"""This module contains a set of functions to handle python protocols for nodes
 where it makes sense.
 """
 
@@ -11,8 +11,8 @@ from __future__ import annotations
 import collections
 import itertools
 import operator as operator_mod
-from collections.abc import Callable, Generator
-from typing import Any
+from collections.abc import Callable, Generator, Iterator, Sequence
+from typing import Any, TypeVar
 
 from astroid import arguments, bases, decorators, helpers, nodes, util
 from astroid.const import Context
@@ -25,17 +25,24 @@ from astroid.exceptions import (
     NoDefault,
 )
 from astroid.nodes import node_classes
-from astroid.typing import ConstFactoryResult
+from astroid.typing import (
+    ConstFactoryResult,
+    InferenceResult,
+    SuccessfulInferenceResult,
+)
 
 raw_building = util.lazy_import("raw_building")
 objects = util.lazy_import("objects")
 
 
-def _reflected_name(name):
+_TupleListNodeT = TypeVar("_TupleListNodeT", nodes.Tuple, nodes.List)
+
+
+def _reflected_name(name) -> str:
     return "__r" + name[2:]
 
 
-def _augmented_name(name):
+def _augmented_name(name) -> str:
     return "__i" + name[2:]
 
 
@@ -78,12 +85,15 @@ _UNARY_OPERATORS: dict[str, Callable[[Any], Any]] = {
 
 
 def _infer_unary_op(obj: Any, op: str) -> ConstFactoryResult:
-    """Perform unary operation on object.
+    """Perform unary operation on `obj`, unless it is `NotImplemented`.
 
     Can raise TypeError if operation is unsupported.
     """
-    func = _UNARY_OPERATORS[op]
-    value = func(obj)
+    if obj is NotImplemented:
+        value = obj
+    else:
+        func = _UNARY_OPERATORS[op]
+        value = func(obj)
     return nodes.const_factory(value)
 
 
@@ -115,12 +125,20 @@ for _KEY, _IMPL in list(BIN_OP_IMPL.items()):
 
 
 @decorators.yes_if_nothing_inferred
-def const_infer_binary_op(self, opnode, operator, other, context, _):
+def const_infer_binary_op(
+    self: nodes.Const,
+    opnode: nodes.AugAssign | nodes.BinOp,
+    operator: str,
+    other: InferenceResult,
+    context: InferenceContext,
+    _: SuccessfulInferenceResult,
+) -> Generator[ConstFactoryResult | type[util.Uninferable], None, None]:
     not_implemented = nodes.Const(NotImplemented)
     if isinstance(other, nodes.Const):
         if (
             operator == "**"
-            and isinstance(self, nodes.Const)
+            and isinstance(self.value, (int, float))
+            and isinstance(other.value, (int, float))
             and (self.value > 1e5 or other.value > 1e5)
         ):
             yield not_implemented
@@ -146,7 +164,12 @@ def const_infer_binary_op(self, opnode, operator, other, context, _):
 nodes.Const.infer_binary_op = const_infer_binary_op
 
 
-def _multiply_seq_by_int(self, opnode, other, context):
+def _multiply_seq_by_int(
+    self: _TupleListNodeT,
+    opnode: nodes.AugAssign | nodes.BinOp,
+    other: nodes.Const,
+    context: InferenceContext,
+) -> _TupleListNodeT:
     node = self.__class__(parent=opnode)
     filtered_elts = (
         helpers.safe_infer(elt, context) or util.Uninferable
@@ -157,7 +180,9 @@ def _multiply_seq_by_int(self, opnode, other, context):
     return node
 
 
-def _filter_uninferable_nodes(elts, context):
+def _filter_uninferable_nodes(
+    elts: Sequence[InferenceResult], context: InferenceContext
+) -> Iterator[SuccessfulInferenceResult]:
     for elt in elts:
         if elt is util.Uninferable:
             yield nodes.Unknown()
@@ -171,13 +196,13 @@ def _filter_uninferable_nodes(elts, context):
 
 @decorators.yes_if_nothing_inferred
 def tl_infer_binary_op(
-    self,
-    opnode: nodes.BinOp,
+    self: _TupleListNodeT,
+    opnode: nodes.AugAssign | nodes.BinOp,
     operator: str,
-    other: nodes.NodeNG,
+    other: InferenceResult,
     context: InferenceContext,
-    method: nodes.FunctionDef,
-) -> Generator[nodes.NodeNG | type[util.Uninferable], None, None]:
+    method: SuccessfulInferenceResult,
+) -> Generator[_TupleListNodeT | nodes.Const | type[util.Uninferable], None, None]:
     """Infer a binary operation on a tuple or list.
 
     The instance on which the binary operation is performed is a tuple
@@ -217,7 +242,14 @@ nodes.List.infer_binary_op = tl_infer_binary_op
 
 
 @decorators.yes_if_nothing_inferred
-def instance_class_infer_binary_op(self, opnode, operator, other, context, method):
+def instance_class_infer_binary_op(
+    self: bases.Instance | nodes.ClassDef,
+    opnode: nodes.AugAssign | nodes.BinOp,
+    operator: str,
+    other: InferenceResult,
+    context: InferenceContext,
+    method: SuccessfulInferenceResult,
+) -> Generator[InferenceResult, None, None]:
     return method.infer_call_result(self, context)
 
 
@@ -227,7 +259,7 @@ nodes.ClassDef.infer_binary_op = instance_class_infer_binary_op
 
 # assignment ##################################################################
 
-"""the assigned_stmts method is responsible to return the assigned statement
+"""The assigned_stmts method is responsible to return the assigned statement
 (e.g. not inferred) according to the assignment type.
 
 The `assign_path` argument is used to record the lhs path of the original node.
@@ -240,7 +272,7 @@ to any intermediary inference necessary.
 
 
 def _resolve_looppart(parts, assign_path, context):
-    """recursive function to resolve multiple assignments on loops"""
+    """Recursive function to resolve multiple assignments on loops."""
     assign_path = assign_path[:]
     index = assign_path.pop(0)
     for part in parts:
@@ -289,14 +321,24 @@ def for_assigned_stmts(
 ) -> Any:
     if isinstance(self, nodes.AsyncFor) or getattr(self, "is_async", False):
         # Skip inferring of async code for now
-        return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+        return {
+            "node": self,
+            "unknown": node,
+            "assign_path": assign_path,
+            "context": context,
+        }
     if assign_path is None:
         for lst in self.iter.infer(context):
             if isinstance(lst, (nodes.Tuple, nodes.List)):
                 yield from lst.elts
     else:
         yield from _resolve_looppart(self.iter.infer(context), assign_path, context)
-    return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+    return {
+        "node": self,
+        "unknown": node,
+        "assign_path": assign_path,
+        "context": context,
+    }
 
 
 nodes.For.assigned_stmts = for_assigned_stmts
@@ -344,7 +386,9 @@ nodes.AssignName.assigned_stmts = assend_assigned_stmts
 nodes.AssignAttr.assigned_stmts = assend_assigned_stmts
 
 
-def _arguments_infer_argname(self, name, context):
+def _arguments_infer_argname(
+    self, name: str | None, context: InferenceContext
+) -> Generator[InferenceResult, None, None]:
     # arguments information may be missing, in which case we can't do anything
     # more
     if not (self.arguments or self.vararg or self.kwarg):
@@ -426,7 +470,7 @@ def arguments_assigned_stmts(
         # reset call context/name
         callcontext = context.callcontext
         context = copy_context(context)
-        context.callcontext = None
+        context.callcontext = callcontext.parent_call_context
         args = arguments.CallSite(callcontext, context=context)
         return args.infer_argument(self.parent, node_name, context)
     return _arguments_infer_argname(self, node_name, context)
@@ -449,7 +493,12 @@ def assign_assigned_stmts(
         self.value.infer(context), assign_path, context
     )
 
-    return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+    return {
+        "node": self,
+        "unknown": node,
+        "assign_path": assign_path,
+        "context": context,
+    }
 
 
 def assign_annassigned_stmts(
@@ -471,7 +520,7 @@ nodes.AugAssign.assigned_stmts = assign_assigned_stmts
 
 
 def _resolve_assignment_parts(parts, assign_path, context):
-    """recursive function to resolve multiple assignments"""
+    """Recursive function to resolve multiple assignments."""
     assign_path = assign_path[:]
     index = assign_path.pop(0)
     for part in parts:
@@ -522,7 +571,12 @@ def excepthandler_assigned_stmts(
             assigned = objects.ExceptionInstance(assigned)
 
         yield assigned
-    return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+    return {
+        "node": self,
+        "unknown": node,
+        "assign_path": assign_path,
+        "context": context,
+    }
 
 
 nodes.ExceptHandler.assigned_stmts = excepthandler_assigned_stmts
@@ -638,7 +692,12 @@ def with_assigned_stmts(
                         context=context,
                     ) from exc
             yield obj
-    return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+    return {
+        "node": self,
+        "unknown": node,
+        "assign_path": assign_path,
+        "context": context,
+    }
 
 
 nodes.With.assigned_stmts = with_assigned_stmts
@@ -651,7 +710,7 @@ def named_expr_assigned_stmts(
     context: InferenceContext | None = None,
     assign_path: list[int] | None = None,
 ) -> Any:
-    """Infer names and other nodes from an assignment expression"""
+    """Infer names and other nodes from an assignment expression."""
     if self.target == node:
         yield from self.value.infer(context=context)
     else:
@@ -667,7 +726,7 @@ nodes.NamedExpr.assigned_stmts = named_expr_assigned_stmts
 
 
 @decorators.yes_if_nothing_inferred
-def starred_assigned_stmts(
+def starred_assigned_stmts(  # noqa: C901
     self: nodes.Starred,
     node: node_classes.AssignedStmtsPossibleNode = None,
     context: InferenceContext | None = None,
@@ -682,6 +741,7 @@ def starred_assigned_stmts(
             A list of indices, where each index specifies what item to fetch from
             the inference results.
     """
+
     # pylint: disable=too-many-locals,too-many-statements
     def _determine_starred_iteration_lookups(
         starred: nodes.Starred, target: nodes.Tuple, lookups: list[tuple[int, int]]
@@ -818,7 +878,6 @@ def starred_assigned_stmts(
         last_lookup = lookup_slice
 
         for element in itered:
-
             # We probably want to infer the potential values *for each* element in an
             # iterable, but we can't infer a list of all values, when only a list of
             # step values are expected:
